/*
 * JoyChuck
 *
 * 2013 http://cubic9.com
 *
 * original sketch is leoJoy
 *   by 2012 Connor,
 *      http://www.imaginaryindustries.com/blog/?p=80
 * nunchuck_funcs.h
 *   by 2008 Tod E. Kurt,
 *      http://todbot.com/blog/2008/02/18/wiichuck-wii-nunchuck-adapter-available/
 *
 */

#include <Wire.h>
#include "nunchuck_funcs.h"

#define DEBUG 0
#define INTERVAL 50

#define STICK_LEFT_DEFAULT 127
#define STICK_RIGHT_DEFAULT 127
#define POV_DEFAULT -1
#define BUTTON_DEFAULT 0

#define PAD_JOYX_MIN 0
#define PAD_JOYX_MAX 255
#define PAD_JOYY_MIN 0
#define PAD_JOYY_MAX 255

#define CHUCK_JOYX_MIN 24
#define CHUCK_JOYX_MAX 230
#define CHUCK_JOYY_MIN 33
#define CHUCK_JOYY_MAX 226

#define SERIAL_BPS 19200

JoyState_t joySt;
int loop_cnt = 0;
byte accx, accy, accz, zbut, cbut, joyx, joyy;

void setup()
{
    #if DEBUG
        Serial.begin(SERIAL_BPS);
    #endif

    nunchuck_init();

    #if DEBUG
        Serial.print("JoyChuck ready\n");
    #endif

    joySt.xAxis = STICK_LEFT_DEFAULT;
    joySt.yAxis = STICK_LEFT_DEFAULT;
    joySt.zAxis = STICK_RIGHT_DEFAULT;
    joySt.xRotAxis = 0;
    joySt.yRotAxis = 0;
    joySt.zRotAxis = 0;
    joySt.throttle = 0;
    joySt.rudder = STICK_RIGHT_DEFAULT;
    joySt.hatSw1 = POV_DEFAULT;
    joySt.hatSw2 = POV_DEFAULT;
    joySt.buttons = BUTTON_DEFAULT;

}


void loop()
{
    if( loop_cnt > INTERVAL ) {
        loop_cnt = 0;

        nunchuck_get_data();

        #if DEBUG
            accx = nunchuck_accelx();
            accy = nunchuck_accely();
            accz = nunchuck_accelz();
        #endif
        zbut = nunchuck_zbutton();
        cbut = nunchuck_cbutton();
        joyx = nunchuck_joyx();
        joyy = nunchuck_joyy();

        #if DEBUG
            Serial.print(" accx:"); Serial.print((byte)accx, DEC);
            Serial.print(" accy:"); Serial.print((byte)accy, DEC);
            Serial.print(" accz:"); Serial.print((byte)accz, DEC);
            Serial.print(" zbut:"); Serial.print((byte)zbut, DEC);
            Serial.print(" cbut:"); Serial.print((byte)cbut, DEC);
            Serial.print(" joyx:"); Serial.print((byte)joyx, DEC);
            Serial.print(" joyy:"); Serial.print((byte)joyy, DEC);
            Serial.println();
        #endif

        joySt.xAxis = constrain(
            map(joyx, CHUCK_JOYX_MIN, CHUCK_JOYX_MAX,
                PAD_JOYX_MIN, PAD_JOYX_MAX),
            PAD_JOYX_MIN, PAD_JOYX_MAX);
        joySt.yAxis = constrain(
            map(joyy, CHUCK_JOYY_MIN, CHUCK_JOYY_MAX,
                PAD_JOYY_MAX, PAD_JOYY_MIN),
            PAD_JOYY_MIN, PAD_JOYY_MAX);
        joySt.buttons = cbut + zbut * 2;

        Joystick.setState(&joySt);
    }
    loop_cnt++;
    delay(1);
}
